@@ -25,7 +25,6 @@ import sys
 import os
 import logging
 import unittest
-import shutil
 
 from ubuntucoreupgrader.upgrader import (
     Upgrader,
@@ -37,7 +36,6 @@ module_dir = os.path.normpath(os.path.realpath(base_dir + os.sep + '..'))
 sys.path.append(base_dir)
 
 from ubuntucoreupgrader.tests.utils import (
-    make_tmp_dir,
     append_file,
     TEST_DIR_MODE,
     create_file,
@@ -68,21 +66,11 @@ def call_upgrader(command_file, root_dir, update):
     args.append(command_file)
     commands = file_to_list(command_file)
 
-    cache_dir = make_tmp_dir()
-
-    def mock_get_cache_dir():
-        cache_dir = update.tmp_dir
-        sys_dir = os.path.join(cache_dir, 'system')
-        os.makedirs(sys_dir, exist_ok=True)
-        return cache_dir
-
     upgrader = Upgrader(parse_args(args), commands, [])
-    upgrader.get_cache_dir = mock_get_cache_dir
+    upgrader.cache_dir = root_dir
     upgrader.MOUNTPOINT_CMD = "true"
     upgrader.run()
 
-    shutil.rmtree(cache_dir)
-
 
 def create_device_file(path, type='c', major=-1, minor=-1):
     '''
@@ -216,7 +204,10 @@ class UpgraderFileRemovalTestCase(UbuntuCoreUpgraderTestCase):
         self.assertTrue(os.path.exists(file_path))
         self.assertTrue(os.path.isfile(file_path))
 
-        call_upgrader(cmd_file, self.victim_dir, self.update)
+        # remove 'system' suffix that upgrader will add back on
+        vdir = os.path.split(self.victim_dir)[0]
+
+        call_upgrader(cmd_file, vdir, self.update)
 
         self.assertFalse(os.path.exists(file_path))
 
@@ -240,7 +231,8 @@ class UpgraderFileRemovalTestCase(UbuntuCoreUpgraderTestCase):
         self.assertTrue(os.path.exists(dir_path))
         self.assertTrue(os.path.isdir(dir_path))
 
-        call_upgrader(cmd_file, self.victim_dir, self.update)
+        vdir = os.path.split(self.victim_dir)[0]
+        call_upgrader(cmd_file, vdir, self.update)
 
         self.assertFalse(os.path.exists(dir_path))
 
@@ -272,7 +264,8 @@ class UpgraderFileRemovalTestCase(UbuntuCoreUpgraderTestCase):
         self.assertTrue(os.path.exists(link_file_path))
         self.assertTrue(os.path.islink(link_file_path))
 
-        call_upgrader(cmd_file, self.victim_dir, self.update)
+        vdir = os.path.split(self.victim_dir)[0]
+        call_upgrader(cmd_file, vdir, self.update)
 
         # original file should still be there
         self.assertTrue(os.path.exists(src_file_path))
@@ -310,7 +303,8 @@ class UpgraderFileRemovalTestCase(UbuntuCoreUpgraderTestCase):
         self.assertTrue(os.path.exists(link_file_path))
         self.assertTrue(os.path.islink(link_file_path))
 
-        call_upgrader(cmd_file, self.victim_dir, self.update)
+        vdir = os.path.split(self.victim_dir)[0]
+        call_upgrader(cmd_file, vdir, self.update)
 
         # original directory should still be there
         self.assertTrue(os.path.exists(src_dir_path))
@@ -352,7 +346,8 @@ class UpgraderFileRemovalTestCase(UbuntuCoreUpgraderTestCase):
 
         self.assertTrue(src_inode == link_inode)
 
-        call_upgrader(cmd_file, self.victim_dir, self.update)
+        vdir = os.path.split(self.victim_dir)[0]
+        call_upgrader(cmd_file, vdir, self.update)
 
         # original file should still be there
         self.assertTrue(os.path.exists(src_file_path))
@@ -392,9 +387,11 @@ class UpgraderFileRemovalTestCase(UbuntuCoreUpgraderTestCase):
         # device because it won't be :)
         self.assertTrue(os.path.isfile(file_path))
 
-        call_upgrader(cmd_file, self.victim_dir, self.update)
+        vdir = os.path.split(self.victim_dir)[0]
+        call_upgrader(cmd_file, vdir, self.update)
 
-        self.assertFalse(os.path.exists(file_path))
+        # upgrader doesn't currently remove device files
+        self.assertTrue(os.path.exists(file_path))
 
 
 class UpgraderFileAddTestCase(UbuntuCoreUpgraderTestCase):
@@ -595,6 +592,67 @@ class UpgraderFileAddTestCase(UbuntuCoreUpgraderTestCase):
         self.assertTrue(is_sym_link_broken(victim_link_file_path))
 
 
+class UpgraderRemoveFileTests(UbuntuCoreUpgraderTestCase):
+
+    def common_removed_file_test(self, contents):
+        '''
+        Common code to test for an invalid removed file.
+
+        The contents parameter specifies the contents of the removed
+        file.
+        '''
+        file = 'created-regular-file'
+
+        create_file(self.update.removed_file, contents)
+
+        file_path = os.path.join(self.update.system_dir, file)
+        create_file(file_path, 'foo bar')
+
+        archive = self.update.create_archive(self.TARFILE)
+        self.assertTrue(os.path.exists(archive))
+
+        cmd_file = os.path.join(self.update.tmp_dir, CMD_FILE)
+        make_command_file(cmd_file, [self.TARFILE])
+
+        vdir = os.path.split(self.victim_dir)[0]
+
+        file_path = os.path.join(vdir, file)
+        self.assertFalse(os.path.exists(file_path))
+
+        # XXX: There is an implicit test here since if the upgrader
+        # fails (as documented on LP: #1437225), this test will also
+        # fail.
+        call_upgrader(cmd_file, vdir, self.update)
+
+        self.assertTrue(os.path.exists(vdir))
+        self.assertTrue(os.path.exists(self.victim_dir))
+        self.assertTrue(os.path.exists(file_path))
+
+        # ensure the empty removed file hasn't removed the directory the
+        # unpack applies to.
+        self.assertTrue(self.victim_dir)
+
+    def test_removed_file_empty(self):
+        '''
+        Ensure the upgrader ignores an empty 'removed' file.
+        '''
+        self.common_removed_file_test('')
+
+    def test_removed_file_space(self):
+        '''
+        Ensure the upgrader handles a 'removed' file containing just a
+        space.
+        '''
+        self.common_removed_file_test(' ')
+
+    def test_removed_file_nl(self):
+        '''
+        Ensure the upgrader handles a 'removed' file containing just a
+        newline
+        '''
+        self.common_removed_file_test('\n')
+
+
 def main():
     kwargs = {}
     format =             \
