@@ -169,9 +169,9 @@ class UpgradeTestCase(unittest.TestCase):
 def touch_file(path):
     '''
     Create an empty file (creating any necessary intermediate
-    directories in @path.
+    directories in @path).
     '''
-    create_file(path, '')
+    create_file(path, None)
 
 
 def make_commands(update_list):
@@ -275,9 +275,70 @@ class UbuntuCoreUpgraderObectTestCase(UbuntuCoreUpgraderTestCase):
             upgrader.TIMESTAMP_FILE = '/dev/null'
             upgrader.MOUNTPOINT_CMD = "true"
             upgrader.run()
+            self.assertTrue(upgrader.other_has_been_formatted)
 
         mock_mkfs.assert_called_with(*MOCK_FS_TUPLE)
 
+    @patch.object(Upgrader, 'sync_partitions')
+    @patch('ubuntucoreupgrader.upgrader.get_mount_details')
+    @patch('ubuntucoreupgrader.upgrader.remount')
+    @patch('ubuntucoreupgrader.upgrader.mount')
+    @patch('ubuntucoreupgrader.upgrader.fsck')
+    @patch('ubuntucoreupgrader.upgrader.mkfs')
+    @patch('ubuntucoreupgrader.upgrader.unmount')
+    def test_mount_unmount_no_format(self, mock_umount, mock_mkfs, mock_fsck,
+                                     mock_mount, mock_remount,
+                                     mock_mount_details, mock_sync_partitions):
+        MOCK_FS_TUPLE = ("device", "fstype", "label")
+        mock_mount_details.return_value = MOCK_FS_TUPLE
+
+        # mkfs should be called if the format command is specified in
+        # the command file.
+        args = ['cmdfile']
+        options = parse_args(args=args)
+        commands = make_commands([self.TARFILE])
+
+        commands.insert(0, 'mount system')
+        commands.append('unmount system')
+
+        upgrader = Upgrader(options, commands, [])
+        upgrader.TIMESTAMP_FILE = '/dev/null'
+        upgrader.MOUNTPOINT_CMD = "true"
+        upgrader.run()
+
+        self.assertTrue(mock_sync_partitions.called)
+
+    @patch.object(Upgrader, 'sync_partitions')
+    @patch('ubuntucoreupgrader.upgrader.get_mount_details')
+    @patch('ubuntucoreupgrader.upgrader.remount')
+    @patch('ubuntucoreupgrader.upgrader.mount')
+    @patch('ubuntucoreupgrader.upgrader.fsck')
+    @patch('ubuntucoreupgrader.upgrader.mkfs')
+    @patch('ubuntucoreupgrader.upgrader.unmount')
+    def test_mount_unmount_with_format(self, mock_umount, mock_mkfs, mock_fsck,
+                                       mock_mount, mock_remount,
+                                       mock_mount_details,
+                                       mock_sync_partitions):
+        MOCK_FS_TUPLE = ("device", "fstype", "label")
+        mock_mount_details.return_value = MOCK_FS_TUPLE
+
+        # mkfs should be called if the format command is specified in
+        # the command file.
+        args = ['cmdfile']
+        options = parse_args(args=args)
+        commands = make_commands([self.TARFILE])
+
+        commands.insert(0, 'format system')
+        commands.insert(1, 'mount system')
+        commands.append('unmount system')
+
+        upgrader = Upgrader(options, commands, [])
+        upgrader.TIMESTAMP_FILE = '/dev/null'
+        upgrader.MOUNTPOINT_CMD = "true"
+        upgrader.run()
+
+        self.assertFalse(mock_sync_partitions.called)
+
     def test_empty_removed_file(self):
         root_dir = make_tmp_dir()
 
@@ -325,6 +386,44 @@ class UbuntuCoreUpgraderObectTestCase(UbuntuCoreUpgraderTestCase):
 
         shutil.rmtree(self.cache_dir)
 
+    def test_other_considered_empty(self):
+
+        si_config_file = '/etc/system-image/channel.ini'
+
+        cache_dir = make_tmp_dir()
+        args = ['cmdfile']
+        options = parse_args(args=args)
+        commands = make_commands([self.TARFILE])
+
+        upgrader = Upgrader(options, commands, [])
+        upgrader.TIMESTAMP_FILE = '/dev/null'
+        upgrader.MOUNTPOINT_CMD = "true"
+        upgrader.cache_dir = cache_dir
+
+        self.assertTrue(upgrader.other_considered_empty())
+
+        target = upgrader.get_mount_target()
+        channel_ini = os.path.normpath('{}/{}'.format(target,
+                                       si_config_file))
+        si_dir = os.path.dirname(channel_ini)
+
+        os.makedirs(si_dir)
+
+        self.assertTrue(upgrader.other_considered_empty())
+
+        touch_file(channel_ini)
+
+        self.assertTrue(upgrader.other_considered_empty())
+
+        create_file(channel_ini, "my size is >0")
+
+        self.assertFalse(upgrader.other_considered_empty())
+
+        os.remove(channel_ini)
+
+        self.assertTrue(upgrader.other_considered_empty())
+
+        shutil.rmtree(cache_dir)
 
 if __name__ == "__main__":
     unittest.main()
